@@ -1,12 +1,16 @@
 package Color::ANSI::Util;
 
-our $DATE = '2018-09-26'; # DATE
-our $VERSION = '0.161'; # VERSION
+our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
+our $DATE = '2020-06-09'; # DATE
+our $DIST = 'Color-ANSI-Util'; # DIST
+our $VERSION = '0.164'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
 
+use Color::RGB::Util qw(rgb_diff);
+
 require Exporter;
 our @ISA       = qw(Exporter);
 our @EXPORT_OK = qw(
@@ -58,9 +62,8 @@ my %ansi16 = (
     15 => 'ffffff',
 );
 my @revansi16;
-for (sort {$a<=>$b} keys %ansi16) {
-    $ansi16{$_} =~ /(..)(..)(..)/;
-    push @revansi16, [hex($1), hex($2), hex($3), $_];
+for my $idx (sort {$a<=>$b} keys %ansi16) {
+    push @revansi16, [$ansi16{$idx}, $idx];
 }
 
 my %ansi256 = (
@@ -109,9 +112,8 @@ my %ansi256 = (
     250 => 'bcbcbc', 251 => 'c6c6c6', 252 => 'd0d0d0', 253 => 'dadada', 254 => 'e4e4e4', 255 => 'eeeeee',
 );
 my @revansi256;
-for (sort {$a<=>$b} keys %ansi256) {
-    $ansi256{$_} =~ /(..)(..)(..)/;
-    push @revansi256, [hex($1), hex($2), hex($3), $_];
+for my $idx (sort {$a<=>$b} keys %ansi256) {
+    push @revansi256, [$ansi256{$idx}, $idx];
 }
 
 $SPEC{ansi16_to_rgb} = {
@@ -174,22 +176,14 @@ sub ansi16_to_rgb {
 sub _rgb_to_indexed {
     my ($rgb, $table) = @_;
 
-    $rgb =~ /^#?([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})$/
-        or die "Invalid RGB input '$rgb'";
-    my $r = hex($1);
-    my $g = hex($2);
-    my $b = hex($3);
-
-    my ($minsqdist, $res);
+    my ($smallest_diff, $res);
     for my $e (@$table) {
-        my $sqdist =
-            abs($e->[0]-$r)**2 + abs($e->[1]-$g)**2 + abs($e->[2]-$b)**2;
+        my $diff = rgb_diff($rgb, $e->[0], 'hsv_hue1');
         # exact match, return immediately
-        return $e->[3] if $sqdist == 0;
-        if (!defined($minsqdist) || $minsqdist > $sqdist) {
-            #say "D:sqdist=$sqdist";
-            $minsqdist = $sqdist;
-            $res = $e->[3];
+        return $e->[1] if $diff == 0;
+        if (!defined($smallest_diff) || $smallest_diff > $diff) {
+            $smallest_diff = $diff;
+            $res = $e->[1];
         }
     }
     return $res;
@@ -420,6 +414,10 @@ our $_color_depth; # cache, can be set during testing
 sub _color_depth {
     unless (defined $_color_depth) {
         {
+            if (exists $ENV{NO_COLOR}) {
+                $_color_depth = 0;
+                last;
+            }
             if (defined $ENV{COLOR} && !$ENV{COLOR}) {
                 $_color_depth = 0;
                 last;
@@ -529,7 +527,14 @@ sub rgb_to_ansi_bg_code {
 
 sub ansibg { goto &rgb_to_ansi_bg_code }
 
-sub ansi_reset { "\e[0m" }
+sub ansi_reset {
+    my $conditional = shift;
+    if ($conditional) {
+        my $cd = _color_depth();
+        return "" if $cd < 16;
+    }
+    "\e[0m";
+}
 
 1;
 # ABSTRACT: Routines for dealing with ANSI colors
@@ -546,7 +551,7 @@ Color::ANSI::Util - Routines for dealing with ANSI colors
 
 =head1 VERSION
 
-This document describes version 0.161 of Color::ANSI::Util (from Perl distribution Color-ANSI-Util), released on 2018-09-26.
+This document describes version 0.164 of Color::ANSI::Util (from Perl distribution Color-ANSI-Util), released on 2020-06-09.
 
 =head1 SYNOPSIS
 
@@ -604,11 +609,13 @@ Arguments ('*' denotes required arguments):
 
 =item * B<$color>* => I<color::ansi16>
 
+
 =back
 
 Return value:  (color::rgb24)
 
 
+
 =head2 ansi256_to_rgb
 
 Usage:
@@ -625,11 +632,13 @@ Arguments ('*' denotes required arguments):
 
 =item * B<$color>* => I<color::ansi256>
 
+
 =back
 
 Return value:  (color::rgb24)
 
 
+
 =head2 rgb_to_ansi16
 
 Usage:
@@ -646,11 +655,13 @@ Arguments ('*' denotes required arguments):
 
 =item * B<$color>* => I<color::rgb24>
 
+
 =back
 
 Return value:  (color::ansi16)
 
 
+
 =head2 rgb_to_ansi16_bg_code
 
 Usage:
@@ -667,11 +678,13 @@ Arguments ('*' denotes required arguments):
 
 =item * B<$color>* => I<color::rgb24>
 
+
 =back
 
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi16_fg_code
 
 Usage:
@@ -688,11 +701,13 @@ Arguments ('*' denotes required arguments):
 
 =item * B<$color>* => I<color::rgb24>
 
+
 =back
 
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi24b_bg_code
 
 Usage:
@@ -709,11 +724,13 @@ Arguments ('*' denotes required arguments):
 
 =item * B<$color>* => I<color::rgb24>
 
+
 =back
 
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi24b_fg_code
 
 Usage:
@@ -730,11 +747,13 @@ Arguments ('*' denotes required arguments):
 
 =item * B<$color>* => I<color::rgb24>
 
+
 =back
 
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi256
 
 Usage:
@@ -751,11 +770,13 @@ Arguments ('*' denotes required arguments):
 
 =item * B<$color>* => I<color::rgb24>
 
+
 =back
 
 Return value:  (color::ansi256)
 
 
+
 =head2 rgb_to_ansi256_bg_code
 
 Usage:
@@ -772,11 +793,13 @@ Arguments ('*' denotes required arguments):
 
 =item * B<$color>* => I<color::rgb24>
 
+
 =back
 
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi256_fg_code
 
 Usage:
@@ -793,11 +816,13 @@ Arguments ('*' denotes required arguments):
 
 =item * B<$color>* => I<color::rgb24>
 
+
 =back
 
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi_bg_code
 
 Usage:
@@ -821,11 +846,13 @@ Arguments ('*' denotes required arguments):
 
 =item * B<$color>* => I<color::rgb24>
 
+
 =back
 
 Return value:  (str)
 
 
+
 =head2 rgb_to_ansi_fg_code
 
 Usage:
@@ -849,10 +876,12 @@ Arguments ('*' denotes required arguments):
 
 =item * B<$color>* => I<color::rgb24>
 
+
 =back
 
 Return value:  (str)
 
+
 =head2 ansi16fg($rgb) => STR
 
 Alias for rgb_to_ansi16_fg_code().
@@ -887,13 +916,23 @@ Alias for rgb_to_ansi_fg_code().
 
 Alias for rgb_to_ansi_bg_code().
 
-=head2 ansi_reset
+=head2 ansi_reset( [ $conditional ])
 
 Returns "\e[0m", which is the ANSI escape sequence to reset color. Normally you
 print this sequence after you print colored text.
 
+If C<$conditional> is set to true, then ansi_reset() will return "" if color is
+disabled.
+
 =head1 ENVIRONMENT
 
+=head2 NO_COLOR
+
+Can be used to explicitly disable color. See L<https://no-color.org> for more
+details.
+
+Observed by: ansi{fg,bg}.
+
 =head2 COLOR => bool
 
 Can be used to explicitly disable color by setting it to 0.
@@ -935,7 +974,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018, 2017, 2016, 2015, 2014, 2013 by perlancar@cpan.org.
+This software is copyright (c) 2020, 2019, 2018, 2017, 2016, 2015, 2014, 2013 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
